// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:io';

import 'package:http/http.dart';
import 'package:wyatt_http_client/src/authentication/interfaces/header_authentication_client.dart';
import 'package:wyatt_http_client/src/utils/authentication_methods.dart';
import 'package:wyatt_http_client/src/utils/header_keys.dart';
import 'package:wyatt_http_client/src/utils/utils.dart';

class BearerAuthenticationClient extends HeaderAuthenticationClient {
  final String token;
  final bool preemptive;

  late String authenticationHeader;
  late String authenticationMethod;

  BearerAuthenticationClient({
    required this.token,
    this.preemptive = true,
    String? authenticationHeader,
    String? authenticationMethod,
    BaseClient? inner,
  }) : super(inner) {
    this.authenticationHeader =
        authenticationHeader ?? HeaderKeys.authorization.toString();
    this.authenticationMethod =
        authenticationMethod ?? AuthenticationMethods.bearer.toString();
  }

  @override
  Map<String, String> modifyHeader(
    Map<String, String> header, [
    BaseRequest? request,
  ]) {
    header[authenticationHeader] = '$authenticationMethod $token';
    return header;
  }

  @override
  Future<StreamedResponse> send(BaseRequest request) async {
    if (preemptive) {
      // Just send request with modified header.
      return super.send(request);
    }

    // Try to send request without modified header,
    final response = await inner.send(request);
    if (response.statusCode == HttpStatus.unauthorized) {
      final newRequest = Utils.copyRequest(request);
      return super.send(newRequest);
    } else {
      return response;
    }
  }
}
